import 'package:flutter_login/src/widgets/animated_button.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutter_login/flutter_login.dart';
import 'package:flutter/material.dart';

// await binding.setSurfaceSize(Size(480, 800));
// // resets the screen to its orinal size after the test end
// addTearDown(() => binding.setSurfaceSize(null));

Widget defaultFlutterLogin() {
  return MaterialApp(
    home: FlutterLogin(
      onSignup: (data) => null,
      onLogin: (data) => null,
      onRecoverPassword: (data) => null,
    ),
  );
}

Finder findNameTextField() {
  return find.byType(TextFormField).at(0);
}

Finder findPasswordTextField() {
  return find.byType(TextFormField).at(1);
}

Finder findConfirmPasswordTextField() {
  return find.byType(TextFormField).at(2);
}

TextField nameTextFieldWidget(WidgetTester tester) {
  return tester.widgetList<TextField>(find.byType(TextField)).elementAt(0);
}

TextField passwordTextFieldWidget(WidgetTester tester) {
  return tester.widgetList<TextField>(find.byType(TextField)).elementAt(1);
}

TextField confirmPasswordTextFieldWidget(WidgetTester tester) {
  return tester.widgetList<TextField>(find.byType(TextField)).elementAt(2);
}

// tester.tap() not working for some reasons. Workaround:
// https://github.com/flutter/flutter/issues/31066#issuecomment-530507319
void clickSubmitButton() {
  final AnimatedButton button =
      find.byType(AnimatedButton).evaluate().first.widget;
  button.onPressed();
}

void clickForgotPasswordButton() {
  final FlatButton button =
      find.byType(FlatButton).evaluate().first.widget;
  button.onPressed();
}

void clickSwitchAuthButton() {
  final FlatButton button =
      find.byType(FlatButton).evaluate().last.widget;
  button.onPressed();
}

void main() {
  final TestWidgetsFlutterBinding binding =
      TestWidgetsFlutterBinding.ensureInitialized();

  testWidgets('Default email validator throws error if not match email regex',
      (WidgetTester tester) async {
    await tester.pumpWidget(defaultFlutterLogin());

    // wait for loading animation to finish
    await tester.pumpAndSettle();

    // empty email
    await tester.enterText(findNameTextField(), '');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    // TODO: put error messages into variables
    expect(nameTextFieldWidget(tester).decoration.errorText, 'Invalid email!');

    // missing '@'
    await tester.enterText(findNameTextField(), 'neargmail.com');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(nameTextFieldWidget(tester).decoration.errorText, 'Invalid email!');

    // missing the part before '@'
    await tester.enterText(findNameTextField(), '@gmail.com');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(nameTextFieldWidget(tester).decoration.errorText, 'Invalid email!');

    // missing the part after '@'
    await tester.enterText(findNameTextField(), 'near@.com');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(nameTextFieldWidget(tester).decoration.errorText, 'Invalid email!');

    // missing domain extension (.com, .org...)
    await tester.enterText(findNameTextField(), 'near@gmail');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(nameTextFieldWidget(tester).decoration.errorText, 'Invalid email!');

    // valid email based on default validator
    await tester.enterText(findNameTextField(), 'near@gmail.com');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(nameTextFieldWidget(tester).decoration.errorText, null);
  });

  testWidgets(
      'Default password validator throws error if password is less than 3 characters',
      (WidgetTester tester) async {
    await tester.pumpWidget(defaultFlutterLogin());

    // wait for loading animation to finish
    await tester.pumpAndSettle();

    // empty
    await tester.enterText(findPasswordTextField(), '');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(passwordTextFieldWidget(tester).decoration.errorText,
        'Password is too short!');

    // too short
    await tester.enterText(findPasswordTextField(), '12');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(passwordTextFieldWidget(tester).decoration.errorText,
        'Password is too short!');

    // valid password based on default validator
    await tester.enterText(findPasswordTextField(), '123');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(passwordTextFieldWidget(tester).decoration.errorText, null);

    // valid password based on default validator
    await tester.enterText(
        findPasswordTextField(), 'aslfjsldfjlsjflsfjslfklsdjflsdjf');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(passwordTextFieldWidget(tester).decoration.errorText, null);
  });

  testWidgets('Confirm password throws error if not match with password',
      (WidgetTester tester) async {
    await tester.pumpWidget(defaultFlutterLogin());

    // wait for loading animation to finish
    await tester.pumpAndSettle();

    // click register button to expand confirm password TextField (hidden when login)
    clickSwitchAuthButton();
    await tester.pumpAndSettle();

    // not match
    await tester.enterText(findPasswordTextField(), 'abcde');
    await tester.pumpAndSettle();
    await tester.enterText(findConfirmPasswordTextField(), 'abcdE');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(passwordTextFieldWidget(tester).decoration.errorText, null);
    expect(confirmPasswordTextFieldWidget(tester).decoration.errorText,
        LoginMessages.defaultConfirmPasswordError);

    // match
    await tester.enterText(findPasswordTextField(), 'abcdE');
    await tester.pumpAndSettle();
    await tester.enterText(findConfirmPasswordTextField(), 'abcdE');
    await tester.pumpAndSettle();
    clickSubmitButton();
    await tester.pumpAndSettle();

    expect(passwordTextFieldWidget(tester).decoration.errorText, null);
    expect(confirmPasswordTextFieldWidget(tester).decoration.errorText, null);
  });

  testWidgets('Hide Logo completely if device height is less than ????',
      (WidgetTester tester) async {
    await tester.pumpWidget(defaultFlutterLogin());
    await tester.pumpAndSettle(Duration(seconds: 1));
    expect(true, true);
  });

  // Test Bugs
  testWidgets('AnimatedTextField: Get transform\'s matrix without applying perspective in the web build',
      (WidgetTester tester) async {
    await tester.pumpWidget(defaultFlutterLogin());
    await tester.pumpAndSettle(Duration(seconds: 1));
    expect(true, true);
  });
}
